package com.example.nto.controller;

import com.example.nto.dto.EmployeeDto;
import com.example.nto.dto.EnterDto;
import com.example.nto.dto.RegisterDto;
import com.example.nto.entity.Code;
import com.example.nto.entity.Employee;
import com.example.nto.entity.Enter;
import com.example.nto.exception.NoRequestBodyException;
import com.example.nto.repository.CodeRepository;
import com.example.nto.repository.EmployeeRepository;
import com.example.nto.service.EmployeeService;
import com.example.nto.service.EnterService;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Optional;


@RestController
@RequiredArgsConstructor
@RequestMapping("/api")
public class EmployeeController {
    private final EmployeeService employeeService;
    private final CodeRepository codeRepository;
    private final EnterService enterService;

    @GetMapping("/login/{login}")
    public ResponseEntity<String> getByUsername(@PathVariable String login) {
        if (login == null) {
            throw new NoRequestBodyException("No or wrong request body!");
        }

        EmployeeDto employeeDto = employeeService.getByLogin(login);
        return ResponseEntity.ok(employeeDto.getLogin());
    }

    @GetMapping("/login")
    public ResponseEntity<EmployeeDto> login(Authentication authentication) {
        if (authentication == null) {
            throw new NoRequestBodyException("No or wrong request body!");
        }
        return ResponseEntity.ok(employeeService.getByLogin(authentication.getName()));
    }

    @GetMapping("/{login}/info")
    public ResponseEntity<EmployeeDto> infoEmployee(@PathVariable String login) {
        EmployeeDto employeeDto = employeeService.getByLogin(login);
        return ResponseEntity.ok(employeeDto);
    }

    @PatchMapping("/{login}/open")
    public ResponseEntity<String> openEmployee(@PathVariable String login, @RequestBody Code code) {
        try {
            EmployeeDto employeeDto = employeeService.getByLogin(login);

            if (employeeDto != null) {
                boolean codeExists = codeRepository.checkCode(code.getValue());

                if (codeExists) {
                    return ResponseEntity.ok("дверь открылась");
                } else {
                    return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("что-то пошло не так");
                }
            } else {
                return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("логина не существует или неверный");
            }
        } catch (Exception e) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("что-то пошло не так");
        }
    }

    @GetMapping("/{login}/enters")
    public ResponseEntity<List<EnterDto>> infoEnters(@PathVariable String login) {
        List<EnterDto> enters = enterService.getByLogin(login);
        return ResponseEntity.ok(enters);
    }
}
